@@ -1,5 +1,5 @@
 /******************************************
-Copyright (c) 2018, Mate Soos <soos.mate@gmail.com>
+Copyright (C) 2009-2020 Authors of CryptoMiniSat, see AUTHORS file <soos.mate@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -55,10 +55,7 @@ lbool CMS_ccnr::main(const uint32_t num_sls_called)
     if (solver->nVars() < 50 ||
         solver->binTri.irredBins + solver->longIrredCls.size() < 10
     ) {
-        if (solver->conf.verbosity) {
-            cout << "c [ccnr] too few variables & clauses"
-            << endl;
-        }
+        verb_print(1, "[ccnr] too few variables & clauses");
         return l_Undef;
     }
     double startTime = cpuTime();
@@ -75,7 +72,7 @@ lbool CMS_ccnr::main(const uint32_t num_sls_called)
 
     vector<bool> phases(solver->nVars()+1);
     for(uint32_t i = 0; i < solver->nVars(); i++) {
-        phases[i+1] = solver->varData[i].polarity;
+        phases[i+1] = solver->varData[i].best_polarity;
     }
 
     int res = ls_s->local_search(&phases, solver->conf.yalsat_max_mems*2*1000*1000);
@@ -225,16 +222,11 @@ struct VarValSorter
 
 vector<pair<uint32_t, double>> CMS_ccnr::get_bump_based_on_cls()
 {
-    if (solver->conf.verbosity) {
-        cout << "c [ccnr] bumping based on clause weights" << endl;
-    }
+    verb_print(1,"[ccnr] bumping based on clause weights");
+
     //Check prerequisites
     assert(toClear.empty());
-    #ifdef SLOW_DEBUG
-    for(const auto x: seen) {
-        assert(x == 0);
-    }
-    #endif
+    SLOW_DEBUG_DO(for(const auto x: seen) assert(x == 0));
 
     vector<pair<uint32_t, double>> tobump_cl_var;
     std::sort(ls_s->_clauses.begin(), ls_s->_clauses.end(), ClWeightSorter());
@@ -249,11 +241,9 @@ vector<pair<uint32_t, double>> CMS_ccnr::get_bump_based_on_cls()
             if (v < solver->nVars() &&
                 solver->varData[v].removed == Removed::none &&
                 solver->value(v) == l_Undef &&
-                seen[v] < solver->conf.sls_bump_var_max_n_times
-            ) {
-                if (seen[v] == 0) {
-                    individual_vars_bumped++;
-                }
+                seen[v] < solver->conf.sls_bump_var_max_n_times)
+            {
+                if (seen[v] == 0) individual_vars_bumped++;
                 seen[v]++;
                 toClear.push_back(Lit(v, false));
                 tobump_cl_var.push_back(std::make_pair(v, 3.0));
@@ -262,10 +252,7 @@ vector<pair<uint32_t, double>> CMS_ccnr::get_bump_based_on_cls()
         }
     }
 
-    //Clear up
-    for(const auto x: toClear) {
-        seen[x.var()] = 0;
-    }
+    for(const auto x: toClear) seen[x.var()] = 0;
     toClear.clear();
 
     return tobump_cl_var;
@@ -301,7 +288,11 @@ vector<pair<uint32_t, double>> CMS_ccnr::get_bump_based_on_conflict_ct()
 
     for(uint32_t i = 1; i < ls_s->_conflict_ct.size(); i++) {
         double val = ls_s->_conflict_ct[i];
-        tobump.push_back(std::make_pair(i-1, (double)val/(double)mymax * 3.0));
+        if (mymax > 0) {
+            tobump.push_back(std::make_pair(i-1, (double)val/(double)mymax * 3.0));
+        } else {
+            tobump.push_back(std::make_pair(i-1, 0));
+        }
 //         if (tobump.back().second > 0) {
 //             cout << "var: " << tobump.back().first << " bump by: " << tobump.back().second << endl;
 //         }
@@ -314,15 +305,15 @@ lbool CMS_ccnr::deal_with_solution(int res, const uint32_t num_sls_called)
     if (solver->conf.sls_get_phase || res) {
         if (solver->conf.verbosity) {
             cout
-            << "c [ccnr] saving best assignment phase"
-            << endl;
+            << "c [ccnr] saving best assignment phase to stable_polar";
+            if (res) cout << " + best_polar";
+            cout << endl;
         }
 
         for(size_t i = 0; i < solver->nVars(); i++) {
-            solver->varData[i].polarity = ls_s->_best_solution[i+1];
+            solver->varData[i].stable_polarity = ls_s->_best_solution[i+1];
             if (res) {
-                solver->varData[i].best_polarity = solver->varData[i].polarity;
-                solver->longest_trail_ever = solver->nVarsOuter();
+                solver->varData[i].best_polarity = ls_s->_best_solution[i+1];
             }
         }
     }
@@ -361,66 +352,18 @@ lbool CMS_ccnr::deal_with_solution(int res, const uint32_t num_sls_called)
             exit(-1);
     }
 
-    for(uint32_t i = 0; i < solver->nVars(); i++) {
-        solver->var_act_vsids[i].offset = 1.0;
-        solver->var_act_maple[i].offset = 1.0;
+
+    for(const auto& v: tobump) solver->bump_var_importance_all(v.first);
+    if (solver->branch_strategy == branch::vsids) {
+        solver->vsids_decay_var_act();
     }
 
-    if (solver->conf.sls_set_offset == 1) {
-        for(const auto& v: tobump) {
-            solver->var_act_vsids[v.first].offset = 1.0+v.second*v.second;
-            solver->var_act_maple[v.first].offset = 1.0+v.second*v.second;
-        }
-    } else if (solver->conf.sls_set_offset == 2) {
-        for(const auto& v: tobump) {
-            solver->var_act_vsids[v.first].offset = 1.0+v.second*v.second*30;
-            solver->var_act_maple[v.first].offset = 1.0+v.second*v.second*30;
-        }
-    } else if (solver->conf.sls_set_offset == 3) {
-        for(const auto& v: tobump) {
-            solver->var_act_vsids[v.first].offset = 1.0+v.second;
-            solver->var_act_maple[v.first].offset = 1.0+v.second;
-        }
-    } else if (solver->conf.sls_set_offset == 4) {
-        for(const auto& v: tobump) {
-            solver->var_act_vsids[v.first].offset = 1.0+v.second*30;
-            solver->var_act_maple[v.first].offset = 1.0+v.second*30;
-        }
-    } else if (solver->conf.sls_set_offset == 5) {
-        for(const auto& v: tobump) {
-            solver->bump_var_importance_all(v.first, true, v.second*30);
-        }
 
-        if (solver->branch_strategy == branch::vsids) {
-            solver->vsids_decay_var_act();
-        }
-    } else if (solver->conf.sls_set_offset == 0) {
-        for(const auto& v: tobump) {
-            solver->bump_var_importance_all(v.first, true, v.second/3.0);
-        }
+    verb_print(1, "[ccnr] Bumped vars: " << tobump.size()
+        << " bump type: " << solver->conf.sls_bump_type);
 
-        if (solver->branch_strategy == branch::vsids) {
-            solver->vsids_decay_var_act();
-        }
-    } else {
-        assert(false && "No such sls_set_offset variabt");
-    }
-
-    if (solver->conf.verbosity) {
-        cout << "c [ccnr] Bumped/set offset to vars: " << tobump.size()
-        << " offset type: " << solver->conf.sls_set_offset
-        << " bump type: " << solver->conf.sls_bump_type
-        << endl;
-    }
-    if (!res) {
-        if (solver->conf.verbosity >= 2) {
-            cout << "c [ccnr] ASSIGNMENT NOT FOUND" << endl;
-        }
-    } else {
-        if (solver->conf.verbosity) {
-            cout << "c [ccnr] ASSIGNMENT FOUND" << endl;
-        }
-    }
+    if (!res) verb_print(2, "[ccnr] ASSIGNMENT NOT FOUND");
+    else verb_print(1, "[ccnr] ASSIGNMENT FOUND");
 
     return l_Undef;
 }
