@@ -1,5 +1,5 @@
 /******************************************
-Copyright (c) 2016, Mate Soos
+Copyright (C) 2009-2020 Authors of CryptoMiniSat, see AUTHORS file
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -34,9 +34,7 @@ THE SOFTWARE.
 #include "searcher.h"
 #include "time_mem.h"
 #include "sqlstats.h"
-#ifdef USE_GAUSS
 #include "gaussian.h"
-#endif
 
 #ifdef USE_VALGRIND
 #include "valgrind/valgrind.h"
@@ -146,6 +144,9 @@ void* ClauseAllocator::allocEnough(
     size += needed;
     currentlyUsedSize += needed;
 
+    #ifdef USE_VALGRIND
+    VALGRIND_MAKE_MEM_UNDEFINED((char*)pointer, neededbytes);
+    #endif
     return pointer;
 }
 
