@@ -21,16 +21,26 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ******************************************************************************/
 
-#ifndef DIMACSPARSER_H
-#define DIMACSPARSER_H
+#pragma once
 
 #include <string.h>
 #include "streambuffer.h"
+#include "solvertypesmini.h"
 #include <cstdlib>
 #include <cmath>
+#include <sstream>
+#include <iostream>
+#include <iomanip>
+#include <vector>
+#include <fstream>
+#include <complex>
+#include <cassert>
 
-using namespace CMSat;
 using std::vector;
+using std::cout;
+using std::endl;
+
+namespace CMSat {
 
 template <class C, class S>
 class DimacsParser
@@ -101,17 +111,6 @@ class DimacsParser
         #endif
 };
 
-#include <sstream>
-#include <iostream>
-#include <iomanip>
-#include <vector>
-#include <fstream>
-#include <complex>
-#include <cassert>
-
-using std::vector;
-using std::cout;
-using std::endl;
 
 template<class C, class S>
 DimacsParser<C, S>::DimacsParser(
@@ -690,4 +689,4 @@ bool DimacsParser<C, S>::parseIndependentSet(C& in)
     return true;
 }
 
-#endif //DIMACSPARSER_H
+}
