@@ -1,6 +1,5 @@
 /*
-Copyright (c) 2010-2015 Mate Soos
-Copyright (c) Kuldeep S. Meel, Daniel J. Fremont
+Copyright (C) 2009-2020 Authors of CryptoMiniSat, see AUTHORS file
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -33,8 +32,7 @@ using namespace CMSat;
 
 extern SATSolver* solverToInterrupt;
 extern int need_clean_exit;
-extern std::string redDumpFname;
-extern std::string irredDumpFname;
+extern double wallclock_time_started;
 void SIGINT_handler(int);
 
 #endif //SIGNALCODE_H_
